//-------------------------------------------------------------------------------------------------+
// Dummy2PosesSender.cpp
// Send two dummy pose datas to port
//-------------------------------------------------------------------------------------------------+

#include "Dummy2PosesSender.h"
#include <iostream>

using namespace std;


//-------------------------------------------------------------------------------------------------+
// Module specification
//-------------------------------------------------------------------------------------------------+
static const char* Dummy2PosesSender_spec[] =
{
    "implementation_id", "Dummy2PosesSender",
    "type_name",         "Dummy2PosesSender",
    "description",       "Send two dummy pose datas to port",
    "version",           "1.0.0",
    "vendor",            "aisl.tut.ac.jp",
    "category",          "dumm",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.start_x", "0.0",
    "conf.default.start_y", "0.0",
    "conf.default.start_heading", "0.0",
    "conf.default.goal_x", "0.0",
    "conf.default.goal_y", "0.0",
    "conf.default.goal_heading", "0.0",
    "conf.default.continuous_output", "0",
    "conf.default.deactivate_to_stop", "1",
    // Widget
    "conf.__widget__.start_x", "text",
    "conf.__widget__.start_y", "text",
    "conf.__widget__.start_heading", "text",
    "conf.__widget__.goal_x", "text",
    "conf.__widget__.goal_y", "text",
    "conf.__widget__.goal_heading", "text",
    "conf.__widget__.continuous_output", "text",
    "conf.__widget__.deactivate_to_stop", "text",
    // Constraints
    ""
};


//-------------------------------------------------------------------------------------------------+
// RXgN^
//-------------------------------------------------------------------------------------------------+
Dummy2PosesSender::Dummy2PosesSender(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    m_start_poseOut("start_pose", m_start_pose),
    m_goal_poseOut("goal_pose", m_goal_pose)
{
	cerr << "------------------------------------------------------------+\n"
		 << " Dummy2PosesSender Component (Ver. 8th December 2011)\n"
		 << "------------------------------------------------------------+\n\n";
}


//-------------------------------------------------------------------------------------------------+
// fXgN^
//-------------------------------------------------------------------------------------------------+
Dummy2PosesSender::~Dummy2PosesSender(){
}


//-------------------------------------------------------------------------------------------------+
// ֐
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t Dummy2PosesSender::onInitialize(){
  
	// Set OutPort buffer
	addOutPort("start_pose", m_start_poseOut);
	addOutPort("goal_pose", m_goal_poseOut);

	// Bind variables and configuration variable
	bindParameter("start_x", m_start_x, "0.0");
	bindParameter("start_y", m_start_y, "0.0");
	bindParameter("start_heading", m_start_heading, "0.0");
	bindParameter("goal_x", m_goal_x, "0.0");
	bindParameter("goal_y", m_goal_y, "0.0");
	bindParameter("goal_heading", m_goal_heading, "0.0");
	bindParameter("continuous_output", m_continuous_output, "0");
	bindParameter("deactivate_to_stop", m_deactivate_to_stop, "1");

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// ANeBx[g̏
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t Dummy2PosesSender::onActivated(RTC::UniqueId ec_id){

	m_start_pose.data.position.x = m_start_x;
	m_start_pose.data.position.y = m_start_y;
	m_start_pose.data.heading    = m_start_heading;

	m_goal_pose.data.length(1);
	m_goal_pose.data[0].position.x = m_goal_x;
	m_goal_pose.data[0].position.y = m_goal_y;
	m_goal_pose.data[0].heading    = m_goal_heading;

	m_start_poseOut.write();
	m_goal_poseOut.write();

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// fBANeBx[g̏
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t Dummy2PosesSender::onDeactivated(RTC::UniqueId ec_id){

	if( m_continuous_output && m_deactivate_to_stop ){

		m_goal_pose.data.length(0);
		m_goal_poseOut.write();
	}

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// s֐
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t Dummy2PosesSender::onExecute(RTC::UniqueId ec_id){

	if( !m_continuous_output ){

		ExecutionContext_var ec = this->get_context(0);
		ec->deactivate_component(this->getObjRef());

		return RTC::RTC_OK;
	}

	m_start_poseOut.write();
	m_goal_poseOut.write();

	return RTC::RTC_OK;
}



//-------------------------------------------------------------------------------------------------+
extern "C"
{
 
  void Dummy2PosesSenderInit(RTC::Manager* manager)
  {
    coil::Properties profile(Dummy2PosesSender_spec);
    manager->registerFactory(profile,
                             RTC::Create<Dummy2PosesSender>,
                             RTC::Delete<Dummy2PosesSender>);
  }
};
